-- License CC0 (Creative Commons license) (c) darkfrei, 2021

function generate_world ()
	world = {}
	world.bird = {x=width/10, y=height/2, vy = 0, jump = 500, radius = 10, vx = 200}
	world.pipes = {}
	world.pipes_default_countdown = 3
	world.pipes_countdown = 0
	world.gravitation = 10^3
	world.score = 0
	world.top_score = 0
	world.n_jumps = 0
	world.n_pipes = 0
	world.current_serie = 0
	
	path = {}
end

function love.load()
	local ddwidth, ddheight = love.window.getDesktopDimensions( display )
	if ddheight > 1080 then
		print('ddheight: ' .. ddheight)
		love.window.setMode(1920, 1080, {resizable=true, borderless=false})
	else
		love.window.setMode(ddwidth, ddheight-200, {resizable=true, borderless=false})
	end
	width, height = love.graphics.getDimensions( )

	
	generate_world ()
end

function new_pipe ()
	local y = (0.2 + 0.6*math.random ())*height
	local s = math.random (2)
	local gap = (s == 1) and 120 - math.random(0, 10*math.log(math.max(1, world.score))) or 120
	local w = (s == 2) and 60 + math.random(0, math.max(0, world.score)) or 60
	
	local pipe = {x = width, w = w, y1=0, h1 = y-gap/2, y2 = y + gap+2, h2 = height-(y + gap+2), 
		collided = false, vx = math.random (0, 50)}
	
	table.insert (world.pipes, pipe)
	
	world.n_pipes = world.n_pipes + 1
end

function update_pipes (dt)
	world.pipes_countdown = world.pipes_countdown - dt
	if world.pipes_countdown <= 0 then
		world.pipes_countdown = world.pipes_default_countdown
		new_pipe ()
	end
	
	
	for i, pipe in pairs (world.pipes) do
		local dx = dt*(world.bird.vx + pipe.vx)
		pipe.x = pipe.x - dx
		if pipe.x + pipe.w < 0 then
			world.pipes[i] = nil
		elseif pipe.x + pipe.w < world.bird.x-world.bird.radius then
			if not (pipe.collided or pipe.passed) then
				world.score = world.score + 1
				world.current_serie = world.current_serie + 1
				world.top_score = math.max (world.score, world.top_score)
				pipe.passed = true
			end
			
		end
	end
end

--https://2dengine.com/?p=intersections#Circle_vs_rectangle
-- start of Circle_vs_rectangle
	local function clamp(n, min, max)
		if n < min then
			n = min
		elseif n > max then
			n = max
		end
		return n
	end
	function circleVsRect(cx, cy, cr, l, t, w, h)
		local dx = clamp(cx, l, l + w) - cx
		local dy = clamp(cy, t, t + h) - cy
		return dx*dx + dy*dy <= cr*cr
	end
-- end of Circle_vs_rectangle

function check_collisions ()
	for i, pipe in pairs (world.pipes) do
		if not pipe.collided then
			if circleVsRect(world.bird.x, world.bird.y, world.bird.radius, pipe.x, pipe.y1, pipe.w, pipe.h1) 
			or circleVsRect(world.bird.x, world.bird.y, world.bird.radius, pipe.x, pipe.y2, pipe.w, pipe.h2) 
			then
				pipe.collided = true
				world.score = math.max (0, world.score - 1)
				world.current_serie = 0
			end
		end
	end
end

function update_bird (dt)
	world.bird.vy = world.bird.vy + dt*world.gravitation
	world.bird.y = world.bird.y + dt*world.bird.vy
--	if world.bird.y > height then
--		world.bird.y = height
--	end
	if world.bird.y > height then
		world.bird.y = 2*height - world.bird.y
		world.bird.vy = -0.5*world.bird.vy
	end
	check_collisions ()
end

function update_path (dt)
	local dx = dt*world.bird.vx
	for i = 1, #path, 2 do
		path[i] = path[i] - dx
		if path[i] < 0 then
			path[i] = nil
			path[i+1] = nil
		end
	end
	
	table.insert (path, 1, world.bird.x)
	table.insert (path, 2, world.bird.y)
end

function love.update(dt)
	update_pipes (dt)
	update_bird (dt)
	update_path (dt)
end

function draw_pipes ()
	for i, pipe in pairs (world.pipes) do
		if pipe.collided then
			love.graphics.setColor(1,0,0)
		else
			love.graphics.setColor(1,1,1)
		end
		love.graphics.rectangle('fill', math.floor(pipe.x), pipe.y1, pipe.w, pipe.h1)
		love.graphics.rectangle('fill', math.floor(pipe.x), pipe.y2, pipe.w, pipe.h2)
	end
end

function draw_bird ()
	love.graphics.circle ('fill', world.bird.x, world.bird.y, world.bird.radius)
end

function draw_path ()
	if #path > 2 then
		love.graphics.line (path)
	end
end



function love.draw()
	draw_pipes ()
	draw_bird ()
	draw_path ()
	
	love.graphics.print ('score: '..world.score, 32,32)
	love.graphics.print ('top score: '..world.top_score, 32,20+32)
	love.graphics.print ('jumps: '..world.n_jumps, 32,2*20+32)
	love.graphics.print ('pipes: '..world.n_pipes, 32,3*20+32)
	love.graphics.print ('current serie: '.. world.current_serie, 32,4*20+32)
	
end

function do_jump ()
	world.n_jumps = world.n_jumps + 1
	world.bird.vy = - world.bird.jump
end

function love.keypressed(key, scancode, isrepeat)
	if key == "space" then
		do_jump ()
	elseif key == "escape" then
		love.event.quit()
	end
end

function love.touchpressed( id, x, y, dx, dy, pressure )
	do_jump ()
end


